@@ -6,9 +6,10 @@
 * @subpackage   XMLElement
 * @author    Andrew McMillan <andrew@mcmillan.net.nz>
 * @copyright Catalyst .Net Ltd, Morphoss Ltd <http://www.morphoss.com/>
-* @license   http://gnu.org/copyleft/gpl.html GNU GPL v2
+* @license   http://www.gnu.org/licenses/lgpl-3.0.txt  GNU LGPL version 3 or later
 */
-require_once("AWLUtilities.php");
+
+require_once('AWLUtilities.php');
 
 /**
 * A class for XML elements which may have attributes, or contain
@@ -17,11 +18,11 @@ require_once("AWLUtilities.php");
 * @package   awl
 */
 class XMLElement {
-  var $tagname;
-  var $xmlns;
-  var $attributes;
-  var $content;
-  var $_parent;
+  protected $tagname;
+  protected $xmlns;
+  protected $attributes;
+  protected $content;
+  protected $_parent;
 
   /**
   * Constructor - nothing fancy as yet.
@@ -31,7 +32,7 @@ class XMLElement {
   * @param array $attributes An array of attribute name/value pairs
   * @param array $xmlns An XML namespace specifier
   */
-  function XMLElement( $tagname, $content=false, $attributes=false, $xmlns=null ) {
+  function __construct( $tagname, $content=false, $attributes=false, $xmlns=null ) {
     $this->tagname=$tagname;
     if ( gettype($content) == "object" ) {
       // Subtree to be parented here
@@ -109,6 +110,7 @@ class XMLElement {
   * @return string The value of that attribute of the element
   */
   function GetAttribute( $attr ) {
+    if ( $attr == 'xmlns' ) return $this->xmlns;
     if ( isset($this->attributes[$attr]) ) return $this->attributes[$attr];
     return null;
   }
@@ -132,15 +134,16 @@ class XMLElement {
   }
 
   /**
-  * Return an array of elements matching the specified tag
+  * Return an array of elements matching the specified tag, or all elements if no tag is supplied.
+  * Unlike GetContent() this will always return an array.
   *
   * @return array The XMLElements within the tree which match this tag
   */
-  function GetElements( $tag, $recursive=false ) {
+  function GetElements( $tag=null, $recursive=false ) {
     $elements = array();
     if ( gettype($this->content) == "array" ) {
       foreach( $this->content AS $k => $v ) {
-        if ( $v->tagname == $tag ) {
+        if ( !isset($tag) || (isset($v->tagname) && $v->tagname == $tag) ) {
           $elements[] = $v;
         }
         if ( $recursive ) {
@@ -148,6 +151,9 @@ class XMLElement {
         }
       }
     }
+    else if ( !isset($tag) || (isset($v->content->tagname) && $this->content->tagname == $tag) ) {
+      $elements[] = $this->content;
+    }
     return $elements;
   }
 
@@ -214,7 +220,7 @@ class XMLElement {
   */
   function &NewElement( $tagname, $content=false, $attributes=false, $xmlns=null ) {
     if ( gettype($this->content) != "array" ) $this->content = array();
-    $element =& new XMLElement($tagname,$content,$attributes,$xmlns);
+    $element = new XMLElement($tagname,$content,$attributes,$xmlns);
     $this->content[] =& $element;
     return $element;
   }
@@ -291,6 +297,11 @@ class XMLElement {
     }
     return $r;
   }
+
+
+  function __tostring() {
+    return $this->Render();
+  }
 }
 
 
