@@ -288,14 +288,16 @@ class vProperty {
   function TestFilter( $filters ) {
     foreach( $filters AS $k => $v ) {
       $tag = $v->GetTag();
+//      dbg_error_log( 'vCalendar', "vProperty:TestFilter: '%s'='%s' => '%s'", $this->name, $tag, $this->content );
       switch( $tag ) {
         case 'urn:ietf:params:xml:ns:caldav:is-defined':
         case 'urn:ietf:params:xml:ns:carddav:is-defined':
+          if ( empty($this->content) ) return false;
           break;
-
+        
         case 'urn:ietf:params:xml:ns:caldav:is-not-defined':
         case 'urn:ietf:params:xml:ns:carddav:is-not-defined':
-          return false;
+          if ( ! empty($this->content) ) return false;
           break;
 
         case 'urn:ietf:params:xml:ns:caldav:time-range':
@@ -307,8 +309,8 @@ class vProperty {
           $search = $v->GetContent();
           $match = $this->TextMatch($search);
           $negate = $v->GetAttribute("negate-condition");
-          if ( isset($negate) && strtolower($negate) == "yes" && $match ) {
-            return false;
+          if ( isset($negate) && strtolower($negate) == "yes" ) {
+            $match = !$match;
           }
           if ( ! $match ) return false;
           break;
@@ -332,14 +334,16 @@ class vProperty {
   function TestParamFilter( $filters, $parameter_value ) {
     foreach( $filters AS $k => $v ) {
       $subtag = $v->GetTag();
+//      dbg_error_log( 'vCalendar', "vProperty:TestParamFilter: '%s'='%s' => '%s'", $this->name, $subtag, $parameter_value );
       switch( $subtag ) {
         case 'urn:ietf:params:xml:ns:caldav:is-defined':
         case 'urn:ietf:params:xml:ns:carddav:is-defined':
+          if ( empty($parameter_value) ) return false;
           break;
 
         case 'urn:ietf:params:xml:ns:caldav:is-not-defined':
         case 'urn:ietf:params:xml:ns:carddav:is-not-defined':
-          return false;
+          if ( ! empty($parameter_value) ) return false;
           break;
 
         case 'urn:ietf:params:xml:ns:caldav:time-range':
@@ -350,10 +354,10 @@ class vProperty {
         case 'urn:ietf:params:xml:ns:caldav:text-match':
           $search = $v->GetContent();
           $match = false;
-          if ( isset($parameter_value) ) $match = strstr( $this->content, $search );
+          if ( !empty($parameter_value) ) $match = strstr( $this->content, $search );
           $negate = $v->GetAttribute("negate-condition");
-          if ( isset($negate) && strtolower($negate) == "yes" && $match ) {
-            return false;
+          if ( isset($negate) && strtolower($negate) == "yes" ) {
+            $match = !$match;
           }
           if ( ! $match ) return false;
           break;
@@ -481,7 +485,7 @@ class vComponent {
           $type = strtoupper($matches[1]);
           $finish = 'END:'.$type;
           $this->type = $type;
-          dbg_error_log( 'vComponent', "::ParseFrom: Start component of type '%s'", $type);
+//          dbg_error_log( 'vComponent', "::ParseFrom: Start component of type '%s'", $type);
         }
         else {
           dbg_error_log( 'vComponent', "::ParseFrom: Ignoring crap before start of component: $line");
@@ -494,7 +498,7 @@ class vComponent {
         if ( $line != "" ) $this->rendered = null;
       }
       else if ( strtoupper($line) == $finish ) {
-        dbg_error_log( 'vComponent', "::ParseFrom: End of component");
+//        dbg_error_log( 'vComponent', "::ParseFrom: End of component");
         $type = null;  // We have reached the end of our component
       }
       else {
@@ -503,13 +507,13 @@ class vComponent {
           $subtype = strtoupper($matches[1]);
           $subfinish = "END:$subtype";
           $subcomponent = $line . "\r\n";
-          dbg_error_log( 'vComponent', "::ParseFrom: Found a subcomponent '%s'", $subtype);
+//          dbg_error_log( 'vComponent', "::ParseFrom: Found a subcomponent '%s'", $subtype);
         }
         else if ( $subtype ) {
           // We are inside a sub-component
           $subcomponent .= $this->WrapComponent($line);
           if ( strtoupper($line) == $subfinish ) {
-            dbg_error_log( 'vComponent', "::ParseFrom: End of subcomponent '%s'", $subtype);
+//            dbg_error_log( 'vComponent', "::ParseFrom: End of subcomponent '%s'", $subtype);
             // We have found the end of a sub-component
             $this->components[] = new vComponent($subcomponent);
             $subtype = false;
@@ -664,7 +668,7 @@ class vComponent {
       $new_prop->Name($new_property);
       $new_prop->Value($value);
       if ( $parameters != null ) $new_prop->Parameters($parameters);
-      dbg_error_log('vComponent'," Adding new property '%s'", $new_prop->Render() );
+//      dbg_error_log('vComponent'," Adding new property '%s'", $new_prop->Render() );
       $this->properties[] = $new_prop;
     }
     else if ( gettype($new_property) ) {
@@ -907,28 +911,47 @@ class vComponent {
   function TestFilter( $filters ) {
     foreach( $filters AS $k => $v ) {
       $tag = $v->GetTag();
+//      dbg_error_log( 'vCalendar', ":TestFilter: '%s' ", $tag );
       switch( $tag ) {
         case 'urn:ietf:params:xml:ns:caldav:is-defined':
         case 'urn:ietf:params:xml:ns:carddav:is-defined':
+          if ( count($this->properties) == 0 && count($this->components) == 0 ) return false;
           break;
-
+        
         case 'urn:ietf:params:xml:ns:caldav:is-not-defined':
         case 'urn:ietf:params:xml:ns:carddav:is-not-defined':
-          return false;
+          if ( count($this->properties) > 0 || count($this->components) > 0 ) return false;
           break;
 
         case 'urn:ietf:params:xml:ns:caldav:comp-filter':
+        case 'urn:ietf:params:xml:ns:carddav:comp-filter':
+          $subcomponents = $this->GetComponents($v->GetAttribute('name'));
           $subfilter = $v->GetContent();
-          $subcomponents = $this->GetComponents($v->GetAttribute("name"));
-          if ( count($subcomponents) > 0 ) {
-            foreach( $subcomponents AS $kk => $subcomponent ) {
-              if ( ! $subcomponent->TestFilter($subfilter) ) return false;
+//          dbg_error_log( 'vCalendar', ":TestFilter: Found '%d' (of %d) subs of type '%s'",
+//                       count($subcomponents), count($this->components), $v->GetAttribute('name') );
+          $subtag = $subfilter[0]->GetTag(); 
+          if ( $subtag == 'urn:ietf:params:xml:ns:caldav:is-not-defined'
+          			 || $subtag == 'urn:ietf:params:xml:ns:carddav:is-not-defined' ) {
+            if ( count($properties) > 0 ) {
+//              dbg_error_log( 'vComponent', ":TestFilter: Wanted none => false" );
+              return false;
+            }
+          }
+          else if ( count($subcomponents) == 0 ) {
+            if ( $subtag == 'urn:ietf:params:xml:ns:caldav:is-defined'
+          			 || $subtag == 'urn:ietf:params:xml:ns:carddav:is-defined' ) {
+//              dbg_error_log( 'vComponent', ":TestFilter: Wanted some => false" );
+              return false;
+            }
+            else {
+//              dbg_error_log( 'vCalendar', ":TestFilter: Wanted something from missing sub-components => false" );
+              $negate = $subfilter[0]->GetAttribute("negate-condition");
+              if ( empty($negate) || strtolower($negate) != 'yes' ) return false;
             }
           }
           else {
-            if ( $subfilter[0] == 'urn:ietf:params:xml:ns:caldav:is-defined'
-                 || $subfilter[0] == 'urn:ietf:params:xml:ns:carddav:is-defined' ) {
-              return false;
+            foreach( $subcomponents AS $kk => $subcomponent ) {
+              if ( ! $subcomponent->TestFilter($subfilter) ) return false;
             }
           }
           break;
@@ -937,15 +960,30 @@ class vComponent {
         case 'urn:ietf:params:xml:ns:caldav:prop-filter':
           $subfilter = $v->GetContent();
           $properties = $this->GetProperties($v->GetAttribute("name"));
-          if ( count($properties) > 0 ) {
-            foreach( $properties AS $kk => $property ) {
-              if ( !$property->TestFilter($subfilter) ) return false;
+          dbg_error_log( 'vCalendar', ":TestFilter: Found '%d' props of type '%s'", count($properties), $v->GetAttribute('name') );
+          $subtag = $subfilter[0]->GetTag();
+          if ( $subtag == 'urn:ietf:params:xml:ns:caldav:is-not-defined'
+          			 || $subtag == 'urn:ietf:params:xml:ns:carddav:is-not-defined' ) {
+            if ( count($properties) > 0 ) {
+//              dbg_error_log( 'vCalendar', ":TestFilter: Wanted none => false" );
+              return false;
+            }
+          }
+          else if ( count($properties) == 0 ) {
+            if ( $subtag == 'urn:ietf:params:xml:ns:caldav:is-defined'
+            			 || $subtag == 'urn:ietf:params:xml:ns:carddav:is-defined' ) {
+//              dbg_error_log( 'vCalendar', ":TestFilter: Wanted some => false" );
+              return false;
+            }
+            else {
+//              dbg_error_log( 'vCalendar', ":TestFilter: Wanted '%s' from missing sub-properties => false", $subtag );
+              $negate = $subfilter[0]->GetAttribute("negate-condition");
+              if ( empty($negate) || strtolower($negate) != 'yes' ) return false;
             }
           }
           else {
-            if ( $subfilter[0] == 'urn:ietf:params:xml:ns:caldav:is-defined'
-                 || $subfilter[0] == 'urn:ietf:params:xml:ns:carddav:is-defined' ) {
-              return false;
+            foreach( $properties AS $kk => $property ) {
+              if ( !$property->TestFilter($subfilter) ) return false;
             }
           }
           break;
